# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.test import TestCase
from datetime import datetime
from . import models
from .test_data import device_mobility
from comm.testcase import PublicBaseClass
# Create your tests here.


class TestDevice_mobilityViewCase(PublicBaseClass):
    @classmethod
    def setUpTestData(cls):
        cls.device = models.Device.objects.create(**device_mobility.device_test_data)
        cls.device_mobility = models.DeviceMobility.objects.create(**device_mobility.device_mobility_test_data)

    def test_device_get(self):
        r = self.operations(url=device_mobility.device_url)
        self.assertEquals(r.data[0].get('name'), device_mobility.device_test_data.get('name'))
        self.assertEquals(r.data[0].get('code'), device_mobility.device_test_data.get('code'))

    def test_device_post(self):
        r = self.operations(url=device_mobility.device_url, method=device_mobility.post_method,
                            params=device_mobility.device_post_data,
                            status_code=device_mobility.created_status_code)
        self.assertEquals(r.data.get('name'), device_mobility.device_post_data.get('name'))
        self.assertEquals(r.data.get('code'), device_mobility.device_post_data.get('code'))

    def test_device_detail(self):
        r = self.operations(url=device_mobility.device_operate_url.format(pk=self.device.id))
        self.assertEquals(r.data.get('name'), device_mobility.device_test_data.get("name"))
        self.assertEquals(r.data.get('code'), device_mobility.device_test_data.get("code"))

    def test_accident_type_put(self):
        r = self.operations(method=device_mobility.put_method, params=device_mobility.device_post_data,
                            url=device_mobility.device_operate_url.format(pk=self.device.id))
        self.assertEquals(r.data.get('name'), device_mobility.device_post_data.get('name'))
        self.assertEquals(r.data.get('code'), device_mobility.device_post_data.get('code'))

    def test_accident_type_patch(self):
        r = self.operations(method=device_mobility.patch_method, params=device_mobility.device_post_data,
                            url=device_mobility.device_operate_url.format(pk=self.device.id))
        self.assertEquals(r.data.get('name'), device_mobility.device_post_data.get('name'))
        self.assertEquals(r.data.get('code'), device_mobility.device_post_data.get('code'))

    def test_accident_type_delete(self):
        self.operations(method=device_mobility.delete_method, status_code=device_mobility.delete_status_code,
                        url=device_mobility.device_operate_url.format(pk=self.device.id))

    def test_device_mobility_post(self):
        r = self.operations(method=device_mobility.post_method, url=device_mobility.device_mobility_url,
                            params=device_mobility.device_mobility_post_data,
                            status_code=device_mobility.created_status_code)
        self.assertEquals(r.data.get('project'), device_mobility.device_mobility_post_data.get('project'))
        self.assertEquals(r.data.get('project_unit'),
                             device_mobility.device_mobility_post_data.get('project_unit'))

    def test_device_mobility_list(self):
        r = self.operations(url=device_mobility.device_mobility_url)
        self.assertEquals(r.data[0].get('project').get('pk'), device_mobility.device_mobility_test_data.get('project').get('pk'))
        self.assertEquals(r.data[0].get('project_unit').get('name'),
                          device_mobility.device_mobility_test_data.get('project_unit').get('name'))

    def test_device_mobility_detail(self):
        r = self.operations(url=device_mobility.device_mobility_operate_url.format(pk=self.device_mobility.id))
        self.assertEquals(r.data.get('project').get('pk'), device_mobility.device_mobility_test_data.get('project').get('pk'))
        self.assertEquals(r.data.get('images'), device_mobility.device_mobility_test_data.get('images'))

    def test_device_mobility_put(self):
        r = self.operations(url=device_mobility.device_mobility_operate_url.format(pk=self.device_mobility.id),
                            method=device_mobility.put_method,
                            params=device_mobility.device_mobility_post_data)
        self.assertEquals(r.data.get('project'), device_mobility.device_mobility_post_data.get('project'))
        self.assertEquals(r.data.get('images'), device_mobility.device_mobility_post_data.get('images'))

    def test_device_mobility_patch(self):
        r = self.operations(url=device_mobility.device_mobility_operate_url.format(pk=self.device_mobility.id),
                            method=device_mobility.patch_method,
                            params=device_mobility.device_mobility_post_data)
        self.assertEquals(r.data.get('project'), device_mobility.device_mobility_post_data.get('project'))
        self.assertEquals(r.data.get('snapshots1'), device_mobility.device_mobility_post_data.get('snapshots1'))

    def test_device_mobility_delete(self):
        self.operations(method=device_mobility.delete_method,
                        url=device_mobility.device_mobility_operate_url.format(pk=self.device_mobility.id),
                        status_code=device_mobility.delete_status_code)

    def test_device_mobility_search(self):
        r = self.operations(url=device_mobility.device_mobility_search_url.format(pro_pk=self.device_mobility.project.get('pk')))
        self.assertEquals(r.data[0].get('project').get('pk'), self.device_mobility.project.get('pk'))
        self.assertEquals(r.data[0].get('project').get('code'), device_mobility.device_mobility_test_data.get('project').get('code'))


import datetime
from rest_framework import serializers
from . import models
from pytz import utc
from django.utils import timezone


class PersonListSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Person
        exclude = ('id',)


class EnterOrLeaveSerializer(serializers.ModelSerializer):
    person = PersonListSerializer()

    class Meta:
        model = models.EnterOrLeave
        fields = '__all__'
        read_only_fields = ('leave_time', 'history', 'create_on', 'enter_status')

    def create(self, validated_data):
        per = models.Person()
        per.id_card = validated_data['person'].get('id_card')
        per.name = validated_data['person'].get('name')
        per.gender = validated_data['person'].get('gender')
        per.birthday = validated_data['person'].get('birthday')
        per.native_place = validated_data['person'].get('native_place')
        per.duty = validated_data['person'].get('duty')
        per.project_unit = validated_data['person'].get('project_unit')
        per.department = validated_data['person'].get('department')
        per.team = validated_data['person'].get('team')
        per.work = validated_data['person'].get('work')
        per.images = validated_data['person'].get('images')
        per.certificate = validated_data['person'].get('certificate')
        per.identity = validated_data['person'].get('identity') if validated_data.get(
            'identity') else 0
        per.remark = validated_data['person'].get('remark')
        per.save()
        per.enterorleave.enter_time = validated_data.get('enter_time').replace(tzinfo=utc) if validated_data.get(
            'enter_time') else timezone.now()
        per.enterorleave.enter_status = 0
        per.enterorleave.create_on = timezone.now()
        per.enterorleave.save()
        return per.enterorleave

    def update(self, instance, validated_data):
        person = instance.person
        person.name = validated_data['person'].get('name') if validated_data['person'].get(
            'name') else person.name
        person.id_card = validated_data['person'].get('id_card') if validated_data['person'].get(
            'id_card') else person.id_card
        person.gender = validated_data['person'].get('gender') if validated_data['person'].get(
            'gender') else person.gender
        person.birthday = validated_data['person'].get('birthday') if validated_data[
            'person'].get('birthday') else person.birthday
        person.native_place = validated_data['person'].get('native_place') if validated_data[
            'person'].get('native_place') else person.native_place
        person.duty = validated_data['person'].get('duty') if validated_data['person'].get(
            'duty') else person.duty
        person.project_unit = validated_data['person'].get('project_unit') if validated_data[
            'person'].get('project_unit') else person.project_unit
        person.department = validated_data['person'].get('department') if validated_data[
            'person'].get('department') else person.department
        person.team = validated_data['person'].get('team') if validated_data['person'].get(
            'team') else person.team
        person.work = validated_data['person'].get('work') if validated_data['person'].get(
            'work') else person.work
        person.images = validated_data['person'].get('images') if validated_data['person'].get(
            'images') else person.images
        person.certificate = validated_data['person'].get('certificate') if validated_data[
            'person'].get('certificate') else person.certificate
        person.identity = validated_data['person'].get('identity') if validated_data[
            'person'].get('identity') else person.identity
        person.remark = validated_data['person'].get('remark') if validated_data['person'].get(
            'remark') else person.remark
        person.save()
        instance.save()
        return instance

    def validate(self, attrs):
        method = self.context.get('request').method if self.context.get('request') else 'POST' 
        if method == 'POST':
            id_card = attrs.get('person', {}).get('id_card')
            if models.EnterOrLeave.objects.filter(person__id_card=id_card):
                raise serializers.ValidationError('id_card already exists')
        return attrs





import datetime
from django.contrib.auth.models import User
from django.test import TestCase, Client
import json, requests
from django.conf import settings
from django.utils.timezone import utc

from . import models
from test_data import person_test_data
from django.utils import timezone
# Create your tests here.


class TestPersonEnterLeave(TestCase):
    def operations(self, method='get', url=None, headers=None, status_code=200, params=None):
        if method == 'get':
            r = self.client.get(url, HTTP_AUTHORIZATION=self.auth)
        elif method == 'post':
            r = self.client.post(url, data=json.dumps(params), content_type='application/json',
                                 HTTP_AUTHORIZATION=self.auth)
        elif method == 'put':
            r = self.client.put(url, data=json.dumps(params), content_type='application/json',
                                HTTP_AUTHORIZATION=self.auth)
        elif method == 'patch':
            r = self.client.patch(url, data=json.dumps(params), content_type='application/json',
                                  HTTP_AUTHORIZATION=self.auth)
        elif method == 'delete':
            r = self.client.delete(url, HTTP_AUTHORIZATION=self.auth)
        else:
            self.assertEqual(method, 'not supported')
        if r.status_code != status_code:
            print r.content
        self.assertEquals(r.status_code, status_code)
        return r

    def setUp(self):
        settings.USE_TZ = False
        user = User.objects.create(username="test")
        user.set_password('123456')
        user.is_staff = True
        user.save()
        self.client.login(username="test", password='123456')
        self.auth = requests.auth._basic_auth_str('test', '123456')
        self.client = Client()

        self.person = models.Person.objects.create(**person_test_data.test_enter_list_data.get('person'))
        self.enter = models.EnterOrLeave.objects.get(person=self.person)
        self.enter.enter_time = timezone.now()
        self.enter.create_on = datetime.datetime.utcnow().replace(tzinfo=utc)
        self.enter.enter_status = 0
        self.enter.save()

    def tearDown(self):
        settings.USE_TZ = True

    def test_person_enter_list(self):
        r = self.operations(url=person_test_data.person_enter_url)
        self.assertEquals(r.data[0].get('enter_status'), 0)
        self.assertEquals(r.data[0].get('person').get('id_card'), person_test_data.test_enter_list_data.get(
            'person').get('id_card'))

    def test_person_enter_list_search(self):
        r = self.operations(url=person_test_data.person_enter_search_url)

    def test_person_enter_post(self):
        r = self.operations(url=person_test_data.person_enter_url, method=person_test_data.post_method,
                            params=person_test_data.test_enter_post_data, status_code=person_test_data.created_status)
        self.assertEquals(r.data.get('person').get('id_card'), person_test_data.test_enter_post_data.get(
            'person').get('id_card'))

    def test_person_enter_put(self):
        r = self.operations(url=person_test_data.person_enter_detail_url.format(pk=self.enter.pk),
                            method=person_test_data.put_method, params=person_test_data.test_enter_post_data)
        self.assertEquals(r.data.get('person').get('name'), person_test_data.test_enter_post_data.get(
            'person').get('name'))

    def test_person_enter_patch(self):
        r = self.operations(url=person_test_data.person_enter_detail_url.format(pk=self.enter.pk),
                            method=person_test_data.patch_method, params=person_test_data.test_enter_post_data)
        self.assertEquals(r.data.get('person').get('name'), person_test_data.test_enter_post_data.get(
            'person').get('name'))

    def test_person_enter_delete(self):
        self.operations(url=person_test_data.person_enter_detail_url.format(pk=self.enter.pk),
                        method=person_test_data.delete_method, status_code=person_test_data.delete_status)

    def test_person_leave(self):
        r = self.operations(url=person_test_data.person_leave_url.format(pk=self.enter.pk))
        self.assertEquals(len(r.data.get('history')), 1)
        self.assertEquals(r.data.get('enter_status'), 1)

    def test_person_enter_bulk(self):
        r = self.operations(url=person_test_data.person_enter_bulk_url, method=person_test_data.post_method,
                            status_code=person_test_data.created_status,
                            params=person_test_data.test_person_enter_bulk_data)
        self.assertEquals(len(r.data), len(person_test_data.test_person_enter_bulk_data))

    def test_enter_leave_calculation(self):
        r = self.operations(url=person_test_data.enter_leave_calculation_url.format(keyword='gender', status=0))
        r = self.operations(url=person_test_data.enter_leave_calculation_url.format(keyword='gender', status=1))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.contrib.postgres.fields import JSONField, HStoreField

# Create your models here.
class Device(models.Model):
    code = models.CharField(max_length=16, unique=True)
    name = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name

class DeviceMobility(models.Model):
    project = JSONField(blank=True, null=True)                      #项目名称
    project_unit = JSONField(blank=True, null=True)                 #单位工程

    construction_org = JSONField(blank=True, null=True)             # 施工单位
    construction_per = JSONField(blank=True, null=True)             # 施工单位负责人
    supervising_org = JSONField(blank=True, null=True)              # 监理单位
    supervising_per = JSONField(blank=True, null=True)              # 监理单位负责人

    device_name = models.ForeignKey(Device, blank=True, null=True)          # 设备名称
    code = models.CharField(max_length=255, blank=True, null=True)          # 设备编号
    number = models.IntegerField(blank=True, null=True)                     # 设备数量
    device_type = models.CharField(max_length=255, blank=True, null=True)   # 设备类型
    device_spec = models.CharField(max_length=255, blank=True, null=True)   # 规格型号
    original_value = models.FloatField(blank=True, null=True)               # 原值
    source = models.IntegerField(blank=True, null=True)                     # 1 自有 2 租赁 3 自带
    manufacturer = JSONField(blank=True, null=True)                         # 厂商
    licence = JSONField(blank=True, null=True)                              # 许可证, 对象
    certificate = JSONField(blank=True, null=True)                          # 合格证, 对象

    attachments = JSONField(blank=True, null=True)                          # 附件
    remark = JSONField(blank=True, null=True)                               # 备注, 预留
    is_ground = models.BooleanField()                                       # 进离场
    images = JSONField(blank=True, null=True)                               # 设备照片
    created_on = models.DateTimeField(auto_now_add=True)                    # 进场时间
    end_on = models.DateTimeField(blank=True, null=True)                    # 离场时间


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests, json
from django.conf import settings
from django.contrib.auth.models import User, Group, Permission

from rest_framework import serializers

from . import models

class DeviceSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.Device
        fields = ('id', 'name', 'code')

class DeviceMobilitySerializer(serializers.ModelSerializer):
    class Meta:
        model = models.DeviceMobility
        fields = ("id", "project", "project_unit", "construction_org", "construction_per",
                  "supervising_org", "supervising_per", "device_name", "code", "number", "device_type",
                  "device_spec", "original_value", "source", "manufacturer", "licence", "end_on",
                  "certificate", "attachments", "remark", "is_ground", "images", "created_on")

class DeviceMobilityDetailSerializer(DeviceMobilitySerializer):
    device_name = DeviceSerializer(many=False)



# -*- coding:utf-8 -*-

from __future__ import unicode_literals

# method
post_method = 'post'
put_method = 'put'
patch_method = 'patch'
delete_method = 'delete'

# response code
created_status_code = 201
delete_status_code = 204

# url
material_list = '/materials/api/material/'
material_detail = '/materials/api/material/{pk}/'
material_spec_list = '/materials/api/material-spec/'
material_spec_detail = '/materials/api/material-spec/{pk}/'
material_entry_list = '/materials/api/material-entry/'
material_entry_detail = '/materials/api/material-entry/{pk}/'

# material create
material_create_data = {
    "name": "钢管",
    "code": "gangguan",
    "unit": "根",
    "material_spec": None

}

# material_spec create
material_spec_create_data = {
    "name": "gg_001",
    "code": "gg_001",
    "material": 1
}

# material_entry create

material_entry_create_data = {
    "project": {  # 项目
        "pk": "4835940668966",
        "code": "QH-V-01",
        "name": "项目名称",
        "obj_type": "C_PJ"
    },
    "project_unit": {  # 单位工程
        "pk": "4836126201382",
        "code": "QH-V-01-01",
        "name": "单位工程名称",
        "obj_type": "C_WP_UNIT"
    },
    "constructionorg": {  # 用料单位
        "pk": "4835940668966",
        "code": "QH-V-01",
        "name": "项目名称",
        "obj_type": "C_PJ"
    },
    "responsor": {  # 进料负责人
        "pk": "4836126201382",
        "code": "QH-V-01-01",
        "name": "单位工程名称",
        "obj_type": "C_WP_UNIT"
    },
    "responsor_org": {  # 进料负责人所属单位
        "pk": "4835940668966",
        "code": "QH-V-01",
        "name": "项目名称",
        "obj_type": "C_PJ"
    },

    "delivery_order": "123465",  # 送货单号
    "lot_number": "1234546",  # 批号
    "remark": {},  # 备注(json格式)
    "material": {  # 物料
        # "code": "aa",  # 必填
        # "name": "aa",
        # "unit": "kg"
    },
    "material_spec": 1,  # 物料规格型号id
    "material_count": 50,  # 物料数量
    "material_value": 0,  # 物料价格
    "material_image": None,  # 物料图片
    "producer": {},  # 生产商
    "supplier": {},  # 供应商
    "location": {},  # 存料地点
    # 进场时间，自动获取当前时间
}

put_data = {
    "project": {
        "pk": "4835940668966",
        "code": "QH-V-01",
        "name": "项目名称",
        "obj_type": "C_PJ"
    },
    "project_unit": {
        "pk": "4836126201382",
        "code": "QH-V-01-01",
        "name": "单位工程名称",
        "obj_type": "C_WP_UNIT"
    },
    "constructionorg": {
        "pk": "4835940668966",
        "code": "QH-V-01",
        "name": "项目名称",
        "obj_type": "C_PJ"
    },
    "responsor": {
        "pk": "4836126201382",
        "code": "QH-V-01-01",
        "name": "单位工程名称",
        "obj_type": "C_WP_UNIT"
    },
    "responsor_org": {
        "pk": "4835940668966",
        "code": "QH-V-01",
        "name": "项目名称",
        "obj_type": "C_PJ"
    },

    "delivery_order": "123465",
    "lot_number": "1234546",
    "remark": {},
    "material":{
        # 'name':"",
        # "code":"",
        # "unit":""
    },
    "material_spec": 1,
    "material_value": 0,
    "material_image": {},
    "producer": {},
    "supplier": {},
    "location": {}

}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.conf.urls import include, url
from . import api_views


urlpatterns = [
    # materials module
    url(r'^api/material/$', api_views.MaterialListView.as_view()),
    url(r'^api/material/(?P<pk>[0-9]+)/$', api_views.MaterialDetailView.as_view()),
    url(r'^api/material-spec/$', api_views.MaterialSpecificationListView.as_view()),
    url(r'^api/material-spec/(?P<pk>[0-9]+)/$', api_views.MaterialSpecificationDetailView.as_view()),
    url(r'^api/material-entry/$', api_views.MaterialEntryListView.as_view()),
    url(r'^api/material-entry/(?P<pk>[0-9]+)/$', api_views.MaterialEntryDetailView.as_view()),



]



from django.conf.urls import url
from . import views

urlpatterns = [
    url(r'api/person-enter/$', views.EnterListView.as_view()),
    url(r'api/person-enter/(?P<pk>[0-9]+)/$', views.EnterDetailView.as_view()),
    url(r'api/person-leave/(?P<pk>[0-9]+)/$', views.PersonLeaveView.as_view()),
    url(r'api/person-enter/bulk/$', views.PersonEnterBulkView.as_view()),
    url(r'api/enter-leave/calculation/', views.PersonEnterLeaveCalculation.as_view()),
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import requests, json
from django.conf import settings
from django.contrib.auth.models import User, Group, Permission
from django.db.models.fields import  related_descriptors

from rest_framework import serializers

from . import models

headers = {'content_type': 'application/json'}

class MaterialSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.Material
        fields = '__all__'

    def to_representation(self, instance):
        data = super(MaterialSerializer, self).to_representation(instance)
        data['material_spec'] = []
        for elem in models.MaterialSpecification.objects.filter(material=instance):
            data['material_spec'].append({'id': elem.id, 'name': elem.name, 'code': elem.code})
        return data



class MaterialSpecificationSerializer(serializers.ModelSerializer):
    material = MaterialSerializer()

    class Meta:
        model = models.MaterialSpecification
        fields = '__all__'


class MaterialSpecificationSerializerEdit(serializers.ModelSerializer):
    class Meta:
        model = models.MaterialSpecification
        fields = '__all__'


class MaterialsEntrySerializer(serializers.ModelSerializer):
    material_spec = MaterialSpecificationSerializerEdit()
    class Meta:
        model = models.MaterialsEntry
        fields = '__all__'

    # def create(self, validated_data):

class MaterialsEntrySerializerEdit(serializers.ModelSerializer):
    class Meta:
        model = models.MaterialsEntry
        fields = '__all__'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from datetime import datetime
from . import models
from .test_data import material_data

from comm.testcase import PublicBaseClass
import json


# Create your tests here.


class TestMaterialViewCase(PublicBaseClass):
    @classmethod
    def setUpTestData(cls):
        cls.material = models.Material.objects.create(**material_data.material_create_data)
        material_data.material_spec_create_data['material'] = cls.material
        cls.material_spec = models.MaterialSpecification.objects.create(**material_data.material_spec_create_data)
        material_data.material_entry_create_data['material_spec'] = cls.material_spec
        cls.material_entry = models.MaterialsEntry.objects.create(**material_data.material_entry_create_data)

    def test_material_list(self):
        # get
        r = self.operations(url=material_data.material_list)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data[0].get('name'), self.material.name)

        # post
        material_data.material_create_data['code'] = 'test'
        r = self.operations(url=material_data.material_list, method=material_data.post_method,
                            params=material_data.material_create_data, status_code=material_data.created_status_code)
        self.assertEqual(r.status_code, material_data.created_status_code)
        self.assertEqual(r.data['code'], 'test')

    def test_material_detail(self):
        url = material_data.material_detail.format(pk=self.material.id)
        # get
        r = self.operations(url=url)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data.get('code'), self.material.code)

        # put
        material_data.material_create_data['name'] = 'test'
        r = self.operations(url=url,
                            method=material_data.put_method, params=material_data.material_create_data)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data['name'], 'test')

        # patch
        patch_param = {
            'name': 'test_01'
        }
        r = self.operations(url=url, method=material_data.patch_method, params=patch_param)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data['name'], 'test_01')

        # delete
        r = self.operations(url=url,
                            method=material_data.delete_method, status_code=material_data.delete_status_code)
        self.assertEqual(r.status_code, material_data.delete_status_code)

    def test_material_spec_list(self):
        url = material_data.material_spec_list
        # get
        r = self.operations(url=url)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data[0].get('name'), self.material_spec.name)

        # post
        material_data.material_spec_create_data['code'] = 'test_spec'
        material_data.material_spec_create_data['material'] = self.material.id
        r = self.operations(url=url, method=material_data.post_method,
                            params=material_data.material_spec_create_data,
                            status_code=material_data.created_status_code)
        self.assertEqual(r.status_code, material_data.created_status_code)
        self.assertEqual(r.data['code'], 'test_spec')

    def test_material_spec_detail(self):
        url = material_data.material_spec_detail.format(pk=self.material_spec.id)
        # get
        r = self.operations(url=url)
        print r.status_code
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data.get('code'), self.material_spec.code)

        # put
        material_data.material_spec_create_data['name'] = 'test_spec'
        r = self.operations(url=url,
                            method=material_data.put_method, params=material_data.material_create_data)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data['name'], 'test')

        # patch
        patch_param = {
            'name': 'test_spec_01'
        }
        r = self.operations(url=url, method=material_data.patch_method, params=patch_param)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data['name'], 'test_spec_01')

        # delete
        r = self.operations(url=url,
                            method=material_data.delete_method, status_code=material_data.delete_status_code)
        self.assertEqual(r.status_code, material_data.delete_status_code)

    def test_material_entry_list(self):
        url = material_data.material_entry_list
        # get
        r = self.operations(url=url)
        self.assertEqual(r.status_code, 200)
        print r.data[0]['material']
        print r.data[0]['material_spec']
        self.assertEqual(r.data[0]['material_spec']['code'], self.material_spec.code)

        # post
        material_data.material_entry_create_data['material_spec'] = self.material_spec.id
        r = self.operations(url=url, method=material_data.post_method,
                            params=material_data.material_entry_create_data,
                            status_code=material_data.created_status_code)
        self.assertEqual(r.status_code, material_data.created_status_code)
        self.assertEqual(r.data['material']['name'], self.material.name)

    def test_material_entry_detail(self):
        url = material_data.material_entry_detail.format(pk=self.material_entry.id)
        # get
        r = self.operations(url=url)
        print r.status_code
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data.get('lot_number'), self.material_entry.lot_number)

        # put
        material_data.put_data['lot_number'] = 'test_number'
        r = self.operations(url=url, method=material_data.put_method, params=material_data.put_data)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data['lot_number'], 'test_number')

        # patch
        patch_param = {
            'material': {
                'name': "",
                'code': "",
                'unit': "",
            },
            'lot_number': 'test_number_01',
            'material_spec': self.material_spec.id
        }
        r = self.operations(url=url, method=material_data.patch_method, params=patch_param)
        self.assertEqual(r.status_code, 200)
        self.assertEqual(r.data['lot_number'], 'test_number_01')

        # delete
        r = self.operations(url=url,
                            method=material_data.delete_method, status_code=material_data.delete_status_code)
        print r.data
        self.assertEqual(r.status_code, material_data.delete_status_code)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from rest_framework.filters import SearchFilter

from django.db import models

class CustomSearchFilter(SearchFilter):
    def filter_queryset(self, request, queryset, view):
        # 全局搜索
        if request.query_params.get(self.search_param):
            return super(CustomSearchFilter, self).filter_queryset(request, queryset, view)

        # 按字段搜索
        else:
            search_params = getattr(view, 'search_params', [])

            for elem in search_params:
                for k,v in elem.items():
                    if request.query_params.get(k):
                        query = models.Q(**{v: request.query_params[k]})
                        queryset = queryset.filter(query)
            return queryset

class CustomSearchFilter1(SearchFilter):
    def filter_queryset(self, request, queryset, view):
        # 全局搜索
        if request.query_params.get(self.search_param):
            return super(CustomSearchFilter1, self).filter_queryset(request, queryset, view)

        # 按字段搜索
        else:
            search_params = getattr(view, 'search_params', [])

            for elem in search_params:
                for k,v in elem.items():
                    if request.query_params.get(k):
                        keyword = request.query_params[k]
                        if v.find('__in') > 0:
                            keyword = keyword.replace(' ', '').split(',')
                        query = models.Q(**{v: keyword})
                        queryset = queryset.filter(query)
            return queryset



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

person_enter_url = '/api/person-enter/'
person_enter_detail_url = '/api/person-enter/{pk}/'
person_leave_url = '/api/person-leave/{pk}/'
person_enter_bulk_url = '/api/person-enter/bulk/'
enter_leave_calculation_url = '/api/enter-leave/calculation/?keyword={keyword}&status={status}'
person_enter_search_url = '/api/person-enter/?id_card=111&name=bbb&gender=ccc&native_place=ddd&enter_status=0&page=1' \
                          '&enter_time_slice=2017-1-1,2018-1-1&department=zzz&project_unit=nnn&team=kkk&identity=0'
post_method = 'post'
put_method = 'put'
patch_method = 'patch'
delete_method = 'delete'

created_status = 201
delete_status = 204

test_enter_list_data = {
        "person": {
            "name": "老王",
            "gender": "男",
            "birthday": "2018-01-01",
            "native_place": "浙江",
            "id_card": "123456",
            "duty": "xxx",
            "project_unit": {},
            "department": {},
            "team": {},
            "work": "",
            "identity": 0,
            "images": {},
            "certificate": {},
            "remark": "xxx"
        }
}

test_enter_post_data = {
        "person": {
            "name": "老王",
            "gender": "男",
            "birthday": "2018-01-01",
            "native_place": "浙江",
            "id_card": "1234567",
            "duty": "xxx",
            "project_unit": {},
            "department": {},
            "team": {},
            "work": "",
            "identity": 0,
            "images": {},
            "certificate": {},
            "remark": "xxx"
        }
}

test_person_enter_bulk_data = [
    {
        "person": {
            "name": "老王",
            "gender": "男",
            "birthday": '1995-02-03',
            "native_place": "新疆",
            "id_card": "12345",
            "duty": "农民",
            "project_unit": {},
            "department": {},
            "team": {},
            "work": "",
            "identity": 0,
            "images": {},
            "certificate": {},
            "remark": "卖羊肉串咯！！"
        }
    },
    {
        "person": {
            "name": "凤姐",
            "gender": "女",
            "birthday": '1998-03-01',
            "native_place": "xxx",
            "id_card": "321654",
            "duty": "网红",
            "project_unit": {},
            "department": {},
            "team": {},
            "work": "",
            "identity": 0,
            "images": {},
            "certificate": {},
            "remark": "sss"
        }
    }

]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.conf.urls import include, url
from . import api_views

urlpatterns = [
    url(r'^api/device-mobility/$', api_views.DeviceMobilityListView.as_view()),
    url(r'^api/device-mobility/(?P<pk>[0-9]+)/$', api_views.DeviceMobilityDetailView.as_view()),
    url(r'^api/device/$', api_views.DeviceListView.as_view()),
    url(r'^api/device/(?P<pk>[0-9]+)/$', api_views.DeviceDetailView.as_view()),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-06-07 09:05

from __future__ import unicode_literals
from django.db import migrations
from django.contrib.postgres.operations import HStoreExtension

class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
    HStoreExtension(),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework.permissions import BasePermission
from rest_framework.compat import is_authenticated

class IsAuthenticated(BasePermission):
    def has_permission(self, request, view):
        from django.conf import settings

        if getattr(settings, 'ENABLE_PERMISSION') and settings.ENABLE_PERMISSION:
            perms_map = getattr(view, 'perms_map', {})
            perms = perms_map.get(request.method, [])
            return request.user and is_authenticated(request.user) \
                and request.user.has_perms(perms)
        else:
            return True



#-*- coding:utf-8 -*-
from __future__ import unicode_literals
import datetime, calendar, requests
from django.db.models import Q
from django.shortcuts import get_object_or_404

from rest_framework import generics, status
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.filters import OrderingFilter
from . import models
from . import serializers
from comm.pagination import StandardPagination
from comm.filter import CustomSearchFilter, CustomSearchFilter1
from comm import permissions

class DeviceMobilityListView(generics.ListCreateAPIView):
    queryset = models.DeviceMobility.objects.all()
    serializer_class = serializers.DeviceMobilitySerializer
    pagination_class = StandardPagination
    filter_backends = (OrderingFilter, CustomSearchFilter1)
    ordering_fields = ('created_on', 'end_on')
    search_params = [
        {'project_pk': 'project__pk__in'},
        {'project_code': 'project__code__in'},
        {'project_unit_pk': 'project_unit__pk__in'},
        {'project_unit_code': 'project_unit__code__in'},
        {'created_year': 'created_on__year__in'},
        {'created_month': 'created_on__month__in'},
        {'created_date': 'created_on__date__in'},
        {'source': 'source__in'},
        {'manufacturer_name': 'manufacturer__name__in'}
    ]

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return serializers.DeviceMobilityDetailSerializer
        else:
            return serializers.DeviceMobilitySerializer

    def get_queryset(self):
        queryset = super(DeviceMobilityListView, self).get_queryset()
        search_key = self.request.GET.get('keyword')
        is_ground = self.request.GET.get('ground')
        if is_ground:
            queryset = queryset.filter(is_ground__exact=is_ground)
        if search_key:
            queryset = queryset.filter(Q(device_name__name__contains=search_key) \
                |Q(device_spec__contains=search_key) \
                |Q(device_type__contains=search_key))
        return queryset

class DeviceMobilityDetailView(generics.RetrieveUpdateDestroyAPIView):
    queryset = models.DeviceMobility.objects.all()
    serializer_class = serializers.DeviceMobilitySerializer
    # permission_classes = (permissions.IsAuthenticated,)
    # perms_map = {
    #     'GET': ['accounts.appmeta.MANAGE.NEWS.READ'],
    #     'PUT': ['accounts.appmeta.MANAGE.NEWS.CREATE'],
    #     'DELETE': ['accounts.appmeta.MANAGE.NEWS.DELETE'],
    # }

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return serializers.DeviceMobilityDetailSerializer
        else:
            return serializers.DeviceMobilitySerializer

    def perform_update(self, serializer):
        instance = serializer.save()
        if serializer.validated_data.get('is_ground') is not True:
            if serializer.validated_data.get('end_on') is None:
                instance.end_on = datetime.datetime.now()
                instance.save()

class DeviceListView(generics.ListCreateAPIView):
    queryset = models.Device.objects.all()
    serializer_class = serializers.DeviceSerializer

class DeviceDetailView(generics.RetrieveUpdateDestroyAPIView):
    queryset = models.Device.objects.all()
    serializer_class = serializers.DeviceSerializer



#-*- coding:utf-8 -*-
from __future__ import unicode_literals
import datetime, calendar, requests
from django.db.models import Q
from django.shortcuts import get_object_or_404

from rest_framework import generics, status
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.permissions import IsAuthenticated, IsAdminUser
from rest_framework.filters import OrderingFilter
from . import models
from . import serializers
from comm.pagination import StandardPagination
from comm.filter import CustomSearchFilter, CustomSearchFilter1
from comm import permissions



class MaterialListView(generics.ListCreateAPIView):
    queryset = models.Material.objects.all()
    serializer_class = serializers.MaterialSerializer
    # permission_classes = (IsAuthenticated,)
    # permission_classes = (permissions.IsAuthenticated,)
    # perms_map = {
    #     'GET': ['accounts.appmeta.MANAGE.NEWS.READ'],
    #     'POST': ['accounts.appmeta.MANAGE.NEWS.WRITE'],
    # }

class MaterialDetailView(generics.RetrieveUpdateDestroyAPIView):
    queryset = models.Material.objects.all()
    serializer_class = serializers.MaterialSerializer
    # permission_classes = (permissions.IsAuthenticated,)
    # perms_map = {
    #     'GET': ['accounts.appmeta.MANAGE.NEWS.READ'],
    #     'PUT': ['accounts.appmeta.MANAGE.NEWS.CREATE'],
    #     'DELETE': ['accounts.appmeta.MANAGE.NEWS.DELETE'],
    # }


class MaterialSpecificationListView(generics.ListCreateAPIView):
    queryset = models.MaterialSpecification.objects.all()
    serializer_class = serializers.MaterialSpecificationSerializer
    # permission_classes = (IsAuthenticated,)
    # permission_classes = (permissions.IsAuthenticated,)
    # perms_map = {
    #     'GET': ['accounts.appmeta.MANAGE.NEWS.READ'],
    #     'POST': ['accounts.appmeta.MANAGE.NEWS.WRITE'],
    # }

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return serializers.MaterialSpecificationSerializer
        else:
            return serializers.MaterialSpecificationSerializerEdit

    def get_queryset(self):
        queryset = super(MaterialSpecificationListView, self).get_queryset()
        material_code = self.request.GET.get('material_code')
        if material_code:
            queryset = queryset.filter(material__code=material_code)
            return queryset
        return queryset


class MaterialSpecificationDetailView(generics.RetrieveUpdateDestroyAPIView):
    queryset = models.MaterialSpecification.objects.all()
    serializer_class = serializers.MaterialSpecificationSerializer
    # permission_classes = (IsAuthenticated,)
    # permission_classes = (permissions.IsAuthenticated,)
    # perms_map = {
    #     'GET': ['accounts.appmeta.MANAGE.NEWS.READ'],
    #     'PUT': ['accounts.appmeta.MANAGE.NEWS.CREATE'],
    #     'DELETE': ['accounts.appmeta.MANAGE.NEWS.DELETE'],
    # }


    def get_serializer_class(self):
        if self.request.method == 'GET':
            return serializers.MaterialSpecificationSerializer
        else:
            return serializers.MaterialSpecificationSerializerEdit

class MaterialEntryListView(generics.ListCreateAPIView):
    queryset = models.MaterialsEntry.objects.all()
    serializer_class = serializers.MaterialsEntrySerializer
    pagination_class = StandardPagination
    # permission_classes = (IsAuthenticated,)
    # permission_classes = (permissions.IsAuthenticated,)
    # perms_map = {
    #     'GET': ['accounts.appmeta.MANAGE.NEWS.READ'],
    #     'POST': ['accounts.appmeta.MANAGE.NEWS.WRITE'],
    # }
    filter_backends = (OrderingFilter, CustomSearchFilter1)
    ordering_fileds = ('create_on')
    search_params = [
        {'project_pk': 'project__pk__in'},
        {'project_code': 'project__code__in'},
        {'project_unit_pk': 'project_unit__pk__in'},
        {'project_unit_code': 'project_unit__code__in'},
        {'producer': 'producer__name__in'},
        {'created_year': 'create_on__year__in'},
        {'created_month': 'create_on__month__in'},
        {'created_date': 'create_on__date__in'},
        {'delivery_order': 'delivery_order__in'},
        {'lot_number': 'lot_number__in'}

    ]

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return serializers.MaterialsEntrySerializer
        else:
            return serializers.MaterialsEntrySerializerEdit

    def get_queryset(self):
        queryset = super(MaterialEntryListView, self).get_queryset()
        search_key = self.request.GET.get('keyword')

        if search_key:
            queryset = queryset.filter(Q(material__name__contains=search_key) \
                | Q(material__code__contains=search_key) \
                | Q(material_spec__code__contains = search_key) \
                | Q(material_spec__name__contains=search_key))
        return queryset

    def perform_create(self, serializer):
        material_spec = serializer.validated_data['material_spec']
        serializer.validated_data['material']['name'] = material_spec.material.name
        serializer.validated_data['material']['code'] = material_spec.material.code
        serializer.validated_data['material']['unit'] = material_spec.material.unit
        serializer.save()




class MaterialEntryDetailView(generics.RetrieveUpdateDestroyAPIView):
    queryset = models.MaterialsEntry.objects.all()
    serializer_class = serializers.MaterialsEntrySerializer

    # permission_classes = (IsAdminUser,)
    # permission_classes = (permissions.IsAuthenticated,)
    # perms_map = {
    #     'GET': ['accounts.appmeta.MANAGE.NEWS.READ'],
    #     'PUT': ['accounts.appmeta.MANAGE.NEWS.CREATE'],
    #     'DELETE': ['accounts.appmeta.MANAGE.NEWS.DELETE'],
    # }

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return serializers.MaterialsEntrySerializer
        else:
            return serializers.MaterialsEntrySerializerEdit

    def perform_update(self, serializer):
        material_spec = serializer.validated_data['material_spec']
        if material_spec:
            serializer.validated_data['material']['name'] = material_spec.material.name
            serializer.validated_data['material']['code'] = material_spec.material.code
            serializer.validated_data['material']['unit'] = material_spec.material.unit
        serializer.save()




import datetime
from django.contrib.postgres.fields import JSONField, ArrayField, HStoreField
from django.db import models
from django.utils import timezone

# Create your models here.
from django.db.models.signals import post_save
from django.dispatch import receiver


class Person(models.Model):
    name = models.CharField(max_length=32)
    gender = models.CharField(max_length=16, null=True, blank=True)
    birthday = models.DateField(null=True, blank=True)
    native_place = models.CharField(max_length=100, null=True, blank=True)
    id_card = models.CharField(max_length=18)
    duty = models.CharField(max_length=100, null=True, blank=True)
    project_unit = HStoreField(blank=True, null=True)
    department = HStoreField(blank=True, null=True)
    team = JSONField(blank=True, null=True)
    work = models.CharField(max_length=100, null=True, blank=True)
    identity = models.IntegerField(default=0)
    images = JSONField(null=True, blank=True)
    certificate = JSONField(null=True, blank=True)
    remark = models.CharField(max_length=255, blank=True, null=True)

    def __unicode__(self):
        return self.name


class EnterOrLeave(models.Model):
    person = models.OneToOneField(Person, on_delete=models.CASCADE)
    enter_time = models.DateTimeField(null=True, blank=True)
    leave_time = models.DateTimeField(null=True, blank=True)
    enter_status = models.IntegerField(default=0)
    create_on = models.DateTimeField(null=True, blank=True)
    history = ArrayField(models.CharField(max_length=200), blank=True, null=True, default=[])

    @receiver(post_save, sender=Person)
    def create_person_enterorleave(sender, instance=None, created=False, **kwargs):
        if created:
            EnterOrLeave.objects.get_or_create(person=instance)














import datetime

from django.db.models import Q, Count
from django.utils import timezone
from django.http import Http404
from pytz import utc
from rest_framework import generics, mixins, permissions
from rest_framework.filters import OrderingFilter
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response
from rest_framework.views import APIView
from . import serializers, models


class PersonEnterPagination(PageNumberPagination):
    page_size = 10
    page_size_query_param = 'page_size'
    page_query_param = "page"
    max_page_size = 100

    def paginate_queryset(self, queryset, request, view=None):
        if request.query_params.get('page'):
            return super(PersonEnterPagination, self).paginate_queryset(queryset, request, view)
        else:
            return None


class EnterListView(generics.ListCreateAPIView):
    serializer_class = serializers.EnterOrLeaveSerializer
    queryset = models.EnterOrLeave.objects.all()
    pagination_class = PersonEnterPagination
    filter_backends = (OrderingFilter,)
    ordering_fields = '__all__'
    ordering = ('id',)

    def get_queryset(self):
        id_card = self.request.query_params.get('id_card')
        name = self.request.query_params.get('name')
        gender = self.request.query_params.get('gender')
        native_place = self.request.query_params.get('native_place')
        identity = self.request.query_params.get('identity')
        enter_status = self.request.query_params.get('enter_status')
        enter_time_slice = self.request.query_params.get('enter_time_slice')
        department = self.request.query_params.get('department')
        project_unit = self.request.query_params.get('project_unit')
        team = self.request.query_params.get('team')
        queryset = models.EnterOrLeave.objects.all()
        if id_card:
            queryset = queryset.filter(person__id_card=id_card)
        if name:
            queryset = queryset.filter(person__name=name)
        if gender:
            queryset = queryset.filter(person__gender=gender)
        if native_place:
            queryset = queryset.filter(person__native_place=native_place)
        if identity:
            queryset = queryset.filter(person__identity=identity)
        if enter_status:
            queryset = queryset.filter(enter_status=enter_status)
        if enter_time_slice:
            slice_list = enter_time_slice.replace(' ', '').split(',')
            if len(slice_list) == 2:
                try:
                    enter_time_from = datetime.datetime.strptime(slice_list[0], "%Y-%m-%d").replace(tzinfo=utc)
                    enter_time_to = datetime.datetime.strptime(slice_list[1], "%Y-%m-%d").replace(tzinfo=utc)
                except Exception as e:
                    raise Http404
                else:
                    queryset = queryset.filter(enter_time__range=(enter_time_from, enter_time_to))
        if project_unit:
            queryset.filter(Q(person__project_unit__pk=project_unit) | Q(person__project_unit__name=project_unit))
        if department:
            queryset.filter(Q(person__department__pk=department) | Q(person__department__name=department))
        if team:
            queryset.filter(Q(person__team__pk=team) | Q(person__team__name=team))

        return queryset

    def get_permissions(self):
        if self.request.method == "GET":
            return []
        else:
            return [permissions.IsAdminUser()]


class EnterDetailView(generics.RetrieveUpdateDestroyAPIView):
    queryset = models.EnterOrLeave.objects.all()
    serializer_class = serializers.EnterOrLeaveSerializer

    def get_object(self):
        try:
            pk = int(self.kwargs['pk'])
            enter_info = models.EnterOrLeave.objects.get(pk=pk)
        except models.EnterOrLeave.DoesNotExist:
            raise Http404
        return enter_info

    def get_permissions(self):
        if self.request.method == 'GET':
            return []
        else:
            return [permissions.IsAdminUser()]

    def perform_destroy(self, instance):
        person = instance.person
        person.delete()
        instance.delete()


class PersonLeaveView(APIView):
    permission_classes = (permissions.IsAdminUser,)

    def get(self, request, pk):
        try:
            leave_person = models.EnterOrLeave.objects.get(pk=pk)
        except models.EnterOrLeave.DoesNotExist:
            raise Http404
        leave_person.leave_time = timezone.now()
        enter_time = leave_person.enter_time.strftime('%Y-%m-%d %H:%M')
        leave_time = leave_person.leave_time.strftime('%Y-%m-%d %H:%M')
        history = "enter_time:{0};leave_time:{1}".format(enter_time, leave_time)
        if not leave_person.enter_status:
            leave_person.history.append(history)
            leave_person.enter_status = 1
        leave_person.save()
        serializer = serializers.EnterOrLeaveSerializer(leave_person)
        return Response(serializer.data)


class PersonEnterBulkView(APIView):
    permission_classes = (permissions.IsAdminUser,)

    def post(self, request):
        res = []
        if type(self.request.data) != list:
            return Response('you should post a list') 
        for index, data in enumerate(self.request.data):
            serializer = serializers.EnterOrLeaveSerializer(data=data)
            if serializer.is_valid():
                serializer.save()
                res.append({"index": index, "status": "seccess"})
            else:
                res.append({"index": index, "error": serializer.errors, "status": "failed"})
        return Response(res, status=201)


class PersonEnterLeaveCalculation(APIView):
    
    def get(self, request):
        keyword = self.request.query_params.get('keyword')
        status = self.request.query_params.get('status', None)
        queryset = models.Person.objects.all()
        if status == 'enter':
            queryset = queryset.filter(enterorleave__enter_status=0)
        if status == 'leave':
            queryset = queryset.filter(enterorleave__enter_status=1)
        if not keyword:
            raise Http404
        return Response(queryset.values(keyword).annotate(count=Count(keyword)))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.contrib.postgres.fields import JSONField, HStoreField, ArrayField

# Create your models here.


class Material(models.Model):
    code = models.CharField(max_length=128, blank=True, null=True, unique=True)
    name = models.CharField(max_length=128, blank=True, null=True)
    unit = models.CharField(max_length=128, blank=True, null=True)
    material_spec = ArrayField(JSONField(null=True, blank=True), null=True, blank=True)
    def __unicode__(self):
        return '{0}-{1}'.format(self.code, self.name)

class MaterialSpecification(models.Model):
    code = models.CharField(max_length=128, blank=True, null=True,  unique=True)
    name = models.CharField(max_length=128, blank=True, null=True)
    material = models.ForeignKey(Material, blank=True, null=True)

    def __unicode__(self):
        return self.name

class MaterialsEntry(models.Model):
    project = JSONField(blank=True, null=True)
    project_unit = JSONField(blank=True, null=True)
    constructionorg = JSONField(blank=True, null=True)
    responsor = JSONField(blank=True, null=True)
    responsor_org = JSONField(blank=True, null=True)
    delivery_order = models.CharField(max_length=128, null=True, blank=True)
    lot_number = models.CharField(max_length=128, null=True, blank=True)
    remark = JSONField(blank=True, null=True)
    material = JSONField(null=True, blank=True)
    material_spec = models.ForeignKey(MaterialSpecification, null=True, blank=True)
    material_count = models.IntegerField(blank=True, null=True, default=1)
    material_value = models.FloatField(blank=True, null=True, default=0.0)
    material_image = JSONField(null=True, blank=True)
    producer = HStoreField(null=True, blank=True)
    supplier = HStoreField(null=True, blank=True)
    location = HStoreField(null=True, blank=True)
    create_on = models.DateTimeField(auto_now_add=True)

    def __unicode__(self):
        return '{0}--{1}'.format(self.material.name, self.material_count)

    class Meta:
        ordering = ['create_on']













from django.contrib.postgres.operations import HStoreExtension
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = []
    operations = [
        HStoreExtension(),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from rest_framework.pagination import PageNumberPagination

class StandardPagination(PageNumberPagination):
    """
    The standard pagination settings for system.
    """
    page_size = 10
    page_size_query_param = 'page_size'
    max_page_size = 100

    def paginate_queryset(self, queryset, request, view=None):
        if request.query_params.get('page'):
            return super(StandardPagination, self).paginate_queryset(
                queryset, request, view)
        else:
            return None


# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.test import TestCase, Client
import json
from django.conf import settings


class PublicBaseClass(TestCase):
    def operations(self, method='get', url=None, headers=None, status_code=200, params=None):
        if method == 'get':
            r = self.client.get(url)
        elif method == 'post':
            r = self.client.post(url, data=json.dumps(params), content_type='application/json')
        elif method == 'put':
            r = self.client.put(url, data=json.dumps(params), content_type='application/json')
        elif method == 'patch':
            r = self.client.patch(url, data=json.dumps(params), content_type='application/json')
        elif method == 'delete':
            r = self.client.delete(url)
        else:
            self.assertEqual(method, 'not supported')
        if r.status_code != status_code:
            print r.data
        self.assertEquals(r.status_code, status_code)
        return r

    def setUp(self):
        settings.USE_TZ = False
        self.client = Client()

    def tearDown(self):
        settings.USE_TZ = True
